@@ -16,26 +16,6 @@ namespace oidn {
     double total;   // maximum progress value
     double current; // current progress value
 
-  public:
-    Progress(ProgressMonitorFunction func, void* userPtr, double total = 1)
-     : func(func),
-       userPtr(userPtr),
-       total(total),
-       current(0)
-    {
-      update();
-    }
-
-    ~Progress()
-    {
-      // Make sure total progress is reported at the end
-      if (current < total)
-      {
-        current = total;
-        update();
-      }
-    }
-
     // Calls the progress monitor function
     void update()
     {
@@ -46,12 +26,33 @@ namespace oidn {
       }
     }
 
+  public:
+    Progress(ProgressMonitorFunction func, void* userPtr, double total = 1)
+     : func(func),
+       userPtr(userPtr),
+       total(total),
+       current(0)
+    {
+      update();
+    }
+
     // Advances the progress with the specified amount and calls the progress monitor function
     void update(double done)
     {
+      assert(done >= 0);
       current = std::min(current + done, total);
       update();
     }
+
+    void finish()
+    {
+      // Make sure total progress is reported at the end
+      if (current < total)
+      {
+        current = total;
+        update();
+      }
+    }
   };
 
 } // namespace oidn
