@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2019 Intel Corporation
+* Copyright 2016-2020 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -74,6 +74,12 @@ private:
 
 protected:
     virtual void SetUp() {
+        data_type = data_traits<data_t>::data_type;
+
+        SKIP_IF(data_type == memory::data_type::s8
+                        && get_test_engine_kind() == engine::kind::gpu,
+                "GPU does not support int8 data type.");
+
         p = ::testing::TestWithParam<decltype(p)>::GetParam();
         catch_expected_failures(
                 [=]() { Test(); }, p.expect_to_fail, p.expected_status);
@@ -83,10 +89,9 @@ protected:
         using bf = normalization_flags;
         p = ::testing::TestWithParam<decltype(p)>::GetParam();
 
-        eng = engine(get_test_engine_kind(), 0);
-        strm = stream(eng);
+        eng = get_test_engine();
+        strm = make_stream(eng);
 
-        memory::data_type data_type = data_traits<data_t>::data_type;
         ASSERT_TRUE(isF32(data_type) || isS8(data_type));
 
         test_bnorm_sizes_t bs = p.sizes;
@@ -138,8 +143,8 @@ protected:
         }
     }
 
-    void Forward(
-            prop_kind pk, normalization_flags flags = (normalization_flags)0u) {
+    void Forward(prop_kind pk,
+            normalization_flags flags = normalization_flags::none) {
         bool useScaleShift
                 = (bool)(flags & normalization_flags::use_scale_shift);
         bool useGlobalStats
@@ -193,8 +198,8 @@ protected:
                 p, src->get(), mean, variance, weights, dst->get(), flags, pk);
     }
 
-    void Backward(
-            prop_kind pk, normalization_flags flags = (normalization_flags)0u) {
+    void Backward(prop_kind pk,
+            normalization_flags flags = normalization_flags::none) {
         bool useScaleShift
                 = (bool)(flags & normalization_flags::use_scale_shift);
 
