@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2019 Intel Corporation
+* Copyright 2016-2020 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -50,7 +50,7 @@ T elu_bwd(T dd, T s, A alpha) {
 }
 
 template <typename T>
-T gelu_fwd(T s) {
+T gelu_tanh_fwd(T s) {
     const float a = 0.797884;
     const float b = 0.044715;
     const float g = a * s * (1 + b * s * s);
@@ -58,7 +58,7 @@ T gelu_fwd(T s) {
 }
 
 template <typename T>
-T gelu_bwd(T dd, T s) {
+T gelu_tanh_bwd(T dd, T s) {
     const float a = 0.797884;
     const float b = 0.044715;
     const float g = a * s * (1 + b * s * s);
@@ -153,6 +153,22 @@ T swish_bwd(T dd, T s, A alpha) {
     return dd * (v + s * alpha * v * (1 - v));
 }
 
+template <typename T>
+T gelu_erf_fwd(T s) {
+    const float sqrt_2_over_2 = 0.707106;
+    float v = s * sqrt_2_over_2;
+    return (T)(sqrt_2_over_2 * v * (1.f + ::erff(v)));
+}
+
+template <typename T>
+T gelu_erf_bwd(T dd, T s) {
+    const float two_over_sqrt_pi = 1.128379;
+    const float sqrt_2_over_2 = 0.707106;
+    float v = s * sqrt_2_over_2;
+    return (T)(dd * 0.5f
+            * (1.f + ::erff(v) + v * two_over_sqrt_pi * ::expf(-v * v)));
+}
+
 struct eltwise_test_params {
     algorithm alg_kind;
     memory::format_tag data_format;
@@ -196,8 +212,9 @@ void check_eltwise_fwd(const eltwise_test_params &p, const memory::desc &md,
             case algorithm::eltwise_soft_relu: ref_d = soft_relu_fwd(s); break;
             case algorithm::eltwise_logistic: ref_d = logistic_fwd(s); break;
             case algorithm::eltwise_exp: ref_d = exp_fwd(s); break;
-            case algorithm::eltwise_gelu: ref_d = gelu_fwd(s); break;
+            case algorithm::eltwise_gelu_tanh: ref_d = gelu_tanh_fwd(s); break;
             case algorithm::eltwise_swish: ref_d = swish_fwd(s, p.alpha); break;
+            case algorithm::eltwise_gelu_erf: ref_d = gelu_erf_fwd(s); break;
             default: assert(!"unknown alg_kind");
         }
         dst_data[i] = ref_d;
@@ -218,10 +235,11 @@ void compare_eltwise_fwd(const eltwise_test_params &p, const memory::desc &md,
                                 == memory::data_type::bf16)
                         ? 5e-2
                         : (p.alg_kind == algorithm::eltwise_elu
-                                  || p.alg_kind == algorithm::eltwise_gelu)
+                                  || p.alg_kind == algorithm::eltwise_gelu_tanh
+                                  || p.alg_kind == algorithm::eltwise_gelu_erf)
                                 ? 2e-5
                                 : p.alg_kind == algorithm::eltwise_soft_relu
-                                        ? 2e-6
+                                        ? 3e-5
                                         : 1e-6);
     compare_data(ref_dst, dst, eps);
 }
@@ -238,11 +256,18 @@ void check_eltwise_bwd(const eltwise_test_params &p, const memory::desc &md,
     const dnnl::impl::memory_desc_wrapper data_mdw(data_d.data);
     const dnnl::impl::memory_desc_wrapper diff_data_mdw(diff_data_d.data);
 
-    const data_t eps = static_cast<data_t>(
-            (p.alg_kind == algorithm::eltwise_soft_relu
-                    || p.alg_kind == algorithm::eltwise_tanh)
-                    ? 2e-6f
-                    : (p.alg_kind == algorithm::eltwise_gelu ? 1e-5f : 1e-6f));
+    float eps_f = 0;
+    if (p.alg_kind == algorithm::eltwise_soft_relu) {
+        eps_f = 2e-6f;
+    } else if (p.alg_kind == algorithm::eltwise_tanh) {
+        eps_f = (get_test_engine_kind() == engine::kind::gpu) ? 2e-5f : 2e-6f;
+    } else if (p.alg_kind == algorithm::eltwise_gelu_tanh
+            || p.alg_kind == algorithm::eltwise_gelu_erf) {
+        eps_f = 1e-5f;
+    } else {
+        eps_f = 1e-6f;
+    }
+    data_t eps = static_cast<data_t>(eps_f);
 
     memory::dim n = n_elems(md);
     for (memory::dim i = 0; i < n; ++i) {
@@ -276,20 +301,27 @@ void check_eltwise_bwd(const eltwise_test_params &p, const memory::desc &md,
                 ref_ds = logistic_bwd(ref_dd, ref_s);
                 break;
             case algorithm::eltwise_exp: ref_ds = exp_bwd(ref_dd, ref_s); break;
-            case algorithm::eltwise_gelu:
-                ref_ds = gelu_bwd(ref_dd, ref_s);
+            case algorithm::eltwise_gelu_tanh:
+                ref_ds = gelu_tanh_bwd(ref_dd, ref_s);
                 break;
             case algorithm::eltwise_swish:
                 ref_ds = swish_bwd(ref_dd, ref_s, p.alpha);
                 break;
+            case algorithm::eltwise_gelu_erf:
+                ref_ds = gelu_erf_bwd(ref_dd, ref_s);
+                break;
             default: assert(!"unknown alg_kind");
         }
 
-        auto tgt = diff_src_data[diff_data_mdw.off_l(i)];
+        data_t tgt = diff_src_data[diff_data_mdw.off_l(i)];
         const data_t diff = tgt == ref_ds ? 0 : tgt - ref_ds;
-        const data_t error = (std::abs(ref_ds) > eps)
+        data_t error = (std::abs(ref_ds) > eps)
                 ? static_cast<data_t>(diff / ref_ds)
                 : diff;
+        if (p.alg_kind == algorithm::eltwise_logistic
+                && (tgt < 1e-3)) { // check for cancellation
+            error = diff;
+        }
         ASSERT_NEAR(error, 0.0, eps);
     }
 }
@@ -321,7 +353,7 @@ protected:
                                 && p.alpha != 0.0))
                         && (data_type == memory::data_type::s32
                                 || data_type == memory::data_type::s8),
-                "DNNL only supports relu w/ slope=0 for integers");
+                "oneDNN only supports relu w/ slope=0 for integers");
         catch_expected_failures(
                 [=]() { Test(); }, p.expect_to_fail, p.expected_status);
     }
@@ -329,11 +361,14 @@ protected:
     void Test() {
         p = ::testing::TestWithParam<eltwise_test_params>::GetParam();
 
-        eng = engine(get_test_engine_kind(), 0);
-        strm = stream(eng);
+        eng = get_test_engine();
+        strm = make_stream(eng);
 
         Forward();
-        Backward();
+        if (data_type == memory::data_type::f32
+                || data_type == memory::data_type::bf16) {
+            Backward();
+        }
     }
 
     void Forward() {
@@ -442,14 +477,15 @@ TEST_P(eltwise_test_s8, TestsEltwise) {}
     }
 
 #define PARAMS_ALL_ALG(...) \
-    EXPAND(PARAMS(eltwise_gelu, __VA_ARGS__)), \
+    EXPAND(PARAMS(eltwise_gelu_tanh, __VA_ARGS__)), \
             EXPAND(PARAMS(eltwise_relu, __VA_ARGS__)), \
             EXPAND(PARAMS(eltwise_tanh, __VA_ARGS__)), \
             EXPAND(PARAMS(eltwise_elu, __VA_ARGS__)), \
             EXPAND(PARAMS(eltwise_square, __VA_ARGS__)), \
             EXPAND(PARAMS(eltwise_abs, __VA_ARGS__)), \
             EXPAND(PARAMS(eltwise_exp, __VA_ARGS__)), \
-            EXPAND(PARAMS(eltwise_swish, __VA_ARGS__))
+            EXPAND(PARAMS(eltwise_swish, __VA_ARGS__)), \
+            EXPAND(PARAMS(eltwise_gelu_erf, __VA_ARGS__))
 
 #define PARAMS_ALL_ALG_SDPART(...) \
     EXPAND(PARAMS(eltwise_linear, __VA_ARGS__)), \
